"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventEmitter = void 0;
var utils_1 = require("./utils");
var EventNames = {
    INITIALIZED: 'initialized',
    NEW_VARIABLES: 'newVariables',
    ERROR: 'error',
    VARIABLE_UPDATED: 'variableUpdated',
    FEATURE_UPDATED: 'featureUpdated',
    CONFIG_UPDATED: 'configUpdated',
    VARIABLE_EVALUATED: 'variableEvaluated',
};
var isInvalidEventKey = function (key) {
    return (!Object.values(EventNames).includes(key) &&
        !key.startsWith(EventNames.VARIABLE_UPDATED) &&
        !key.startsWith(EventNames.FEATURE_UPDATED) &&
        !key.startsWith(EventNames.NEW_VARIABLES) &&
        !key.startsWith(EventNames.VARIABLE_EVALUATED));
};
var EventEmitter = /** @class */ (function () {
    function EventEmitter() {
        this.handlers = {};
    }
    EventEmitter.prototype.subscribe = function (key, handler) {
        (0, utils_1.checkParamType)('key', key, 'string');
        (0, utils_1.checkParamType)('handler', handler, 'function');
        if (isInvalidEventKey(key)) {
            throw new Error('Not a valid event to subscribe to');
        }
        if (!this.handlers[key]) {
            this.handlers[key] = [handler];
        }
        else {
            this.handlers[key].push(handler);
        }
    };
    EventEmitter.prototype.unsubscribe = function (key, handler) {
        (0, utils_1.checkParamType)('key', key, 'string');
        if (isInvalidEventKey(key)) {
            return;
        }
        if (handler) {
            var handlerIndex = this.handlers[key].findIndex(function (h) { return h === handler; });
            this.handlers[key].splice(handlerIndex, 1);
        }
        else {
            this.handlers[key] = [];
        }
    };
    EventEmitter.prototype.emit = function (key) {
        var _a;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        (0, utils_1.checkParamType)('key', key, 'string');
        (_a = this.handlers[key]) === null || _a === void 0 ? void 0 : _a.forEach(function (handler) {
            new Promise(function (resolve) {
                handler.apply(void 0, args);
                resolve(true);
            });
        });
    };
    EventEmitter.prototype.emitInitialized = function (success) {
        this.emit(EventNames.INITIALIZED, success);
    };
    EventEmitter.prototype.emitError = function (error) {
        this.emit(EventNames.ERROR, error);
    };
    EventEmitter.prototype.emitConfigUpdate = function (newVariableSet) {
        this.emit(EventNames.CONFIG_UPDATED, newVariableSet);
    };
    EventEmitter.prototype.emitVariableEvaluated = function (variable) {
        this.emit("".concat(EventNames.VARIABLE_EVALUATED, ":*"), variable.key, variable);
        this.emit("".concat(EventNames.VARIABLE_EVALUATED, ":").concat(variable.key), variable.key, variable);
    };
    EventEmitter.prototype.emitVariableUpdates = function (oldVariableSet, newVariableSet, variableDefaultMap) {
        var _this = this;
        var keys = new Set(Object.keys(oldVariableSet).concat(Object.keys(newVariableSet)));
        var newVariables = false;
        keys.forEach(function (key) {
            var oldVariableValue = oldVariableSet[key] && oldVariableSet[key].value;
            var newVariable = newVariableSet[key];
            var newVariableValue = newVariable && newVariableSet[key].value;
            if (JSON.stringify(oldVariableValue) !==
                JSON.stringify(newVariableValue)) {
                var variables = variableDefaultMap[key] &&
                    Object.values(variableDefaultMap[key]);
                if (variables) {
                    newVariables = true;
                    variables.forEach(function (variable) {
                        var _a;
                        variable.value =
                            newVariableValue !== null && newVariableValue !== void 0 ? newVariableValue : variable.defaultValue;
                        variable.isDefaulted =
                            newVariableValue === undefined ||
                                newVariableValue === null;
                        (_a = variable.callback) === null || _a === void 0 ? void 0 : _a.call(variable, variable.value);
                    });
                }
                var finalVariable = newVariable || null;
                _this.emit("".concat(EventNames.VARIABLE_UPDATED, ":*"), key, finalVariable);
                _this.emit("".concat(EventNames.VARIABLE_UPDATED, ":").concat(key), key, finalVariable);
            }
        });
        if (newVariables) {
            this.emit("".concat(EventNames.NEW_VARIABLES));
        }
    };
    EventEmitter.prototype.emitFeatureUpdates = function (oldFeatureSet, newFeatureSet) {
        var _this = this;
        var keys = Object.keys(oldFeatureSet).concat(Object.keys(newFeatureSet));
        keys.forEach(function (key) {
            var oldFeatureVariation = oldFeatureSet[key] && oldFeatureSet[key]._variation;
            var newFeature = newFeatureSet[key];
            var newFeatureVariation = newFeature && newFeatureSet[key]._variation;
            var finalFeature = newFeature || null;
            if (oldFeatureVariation !== newFeatureVariation) {
                _this.emit("".concat(EventNames.FEATURE_UPDATED, ":*"), key, finalFeature);
                _this.emit("".concat(EventNames.FEATURE_UPDATED, ":").concat(key), key, finalFeature);
            }
        });
    };
    return EventEmitter;
}());
exports.EventEmitter = EventEmitter;
//# sourceMappingURL=EventEmitter.js.map